@@ -59,9 +59,16 @@ def createData(rows=20, cols=5):
             i+=1
     return data
 
-def GUITest(root):
-    """Setup a table and populate it with data"""
+def createTable(model):
+    t=Toplevel()
+    app = App(t)
+    master = app.main
+    table = TableCanvas(master, model,rowheaderwidth=50)
+    table.createTableFrame()
+    return table
 
+def test1(root):
+    """Setup a table and populate it with data"""
     app = App(root)
     master = app.main
     model = TableModel()
@@ -69,23 +76,23 @@ def GUITest(root):
     #import after model created
     #print data
     model.importDict(data)
-    from newtable import LargeTable
     table = TableCanvas(master, model,
                         cellwidth=60, cellbackgr='#e3f698',
-                        thefont=('Arial',12),rowheight=18, rowheaderwidth=50,
+                        thefont=('Arial',12),rowheight=18, rowheaderwidth=30,
                         rowselectedcolor='yellow', editable=True)
     table.createTableFrame()
+    #table.sortTable(columnName='label')
     #remove cols
     model.deleteColumns([0])
     model.deleteRows(range(0,2))
-    table.redrawTable()
+    #table.redrawTable()
     #add rows and cols
     table.addRow(1,label='aaazzz')
     table.addRow(label='bbb')
     table.addRow(**{'label':'www'})
     table.addColumn('col6')
     model.data[1]['col6']='TEST'
-    table.redrawTable()
+    #table.redrawTable()
     #change col labels
     model.columnlabels['col6'] = 'new label'
     #set and get selections
@@ -93,18 +100,73 @@ def GUITest(root):
     table.setSelectedCol(1)
     table.setSelectedCells(1,80,2,4)
     #print table.getSelectionValues()
-    #table.plot_Selected(graphtype='XY')
+    #table.plotSelected(graphtype='XY')
     #save data
+    #table.addRows(50000)
     model.save('test.table')
     #load new data
     table.load('test.table')
-    print 'GUI tests done'
     #root.after(2000, root.quit)
     return
 
-def main():
+def test2():
+    """Multuple tables in one window"""
+    t=Toplevel()
+    app = App(t)
+    master = app.main
+    c=0; r=1
+    for i in range(12):
+        model = TableModel()
+        data = createData(50)
+        model.importDict(data)
+        fr = Frame(master)
+        if c%3==0: c=0; r+=1
+        fr.grid(row=r,column=c,sticky='nws')
+        table = TableCanvas(fr, model, width=250,height=150,rowheaderwidth=0)
+        table.createTableFrame()
+        c+=1
+    return
+
+def test3():
+    """Drawing large tables"""
+    data = createData(10000)
+    model = TableModel()
+    model.importDict(data)
+    createTable(model)
+    return
+
+def test4():
+    """Filtering/searching"""
+    model = TableModel()
+    data = createData(100)
+    model.importDict(data)
+    model.addColumn('comment')
+    for i in model.reclist:
+        val = random.sample(['a','b','c'],1)[0]
+        model.data[i]['comment'] = val
+    #searchterms = [('label', 'aa', 'contains', 'AND'),
+    #               ('label', 'bb', 'contains', 'OR')]
+    searchterms = [('comment', 'a', '!=', 'AND'),
+                   ('comment', 'b', '!=', 'AND')]
+    vals = model.getColumnData(columnIndex=0, filters=searchterms)
+    #model.getColumns(model.columnNames, filters=searchterms)
+    #model.getDict(model.columnNames, filters=searchterms)
+    print '%s found' %len(vals)
+    #createTable(model)
+    return
+
+def GUITests():
+    """Run standard tests"""
     root = Tk()
-    GUITest(root)
+    test1(root)
+    test2()
+    test3()
+    test4()
+    print 'GUI tests done'
+    return root
+
+def main():
+    root = GUITests()
     root.mainloop()
     #loadSaveTest()
 
